C***********************************************************************
C*                       SKYNEUT (version 1.1)                         *
C*                                                                     *
C*  This program evaluates the neutron and sec. gamma skyshine dose    *
C*  due to a isotropic, point neutron source collimated by three       *
C*  simple geometries: an open silo, a vertical black wall, and a      *
C*  rectangular building.                                              *
C*                                                                     *
C*  The calculation is based on the integral line-beam method using    *
C*  an improved 3-parameter approximation for the line-beam response   *
C*  function.                                                          *
C*                                                                     *
C*  Source energy E must be  0.01 <= E <= 14 MeV.                      *
C*                                                                     *
C*  References for the calculational methods are:                      *
C*  (1) J.K. Shultis, R.E. Faw and M.S. Bassett, "The Integral Line-   *
C*      Beam Method for Gamma Skyshine Analysis," Nucl. Sci. Engg.,    *
C*      107, 228-245 (1991).                                           *
C*  (2) J.K. Shultis, and R.E. Faw, "Extensions to the Integral Line-  *
C*      Beam Method for Gamma-Ray Skyshine Analyses," SAND-xxxx,       *
C*      Sandia National Lab., Albuquerque, NM, 1994.                   *
C*                                                                     *
C*  Needed Subroutines:                                                *
C*      XBEAM   Evaluates the LBRF from 3-parameter approxmation       *
C*      XBINI   Reads parameters for the LBRF approximation            *
C*      EAGRID  Creates energy and angular grid for the LBRF approx.   *
C*      SILOIO  Reads input data and outputs results for silo problem  *
C*      SILO    Calculates skyshine dose for silo geometry             *
C*      WALLIO  Reads input data and outputs results for wall problem  *
C*      WALL    Calculates skyshine dose for wall geometry             *
C*      BOXIO   Reads input data and outputs results for silo problem  *
C*      BOX     Calculates skyshine dose for silo geometry             *
C*      READGS  Gives Gaussian quadrature ordinates and weights        *
C*                                                                     *
C*  Data Files Needed:  NEDE.DAT, NDEP.DAT, NDEI.DAT  and GEDE.DAT     *
C*                      GDEP.DAT, GDEI.DAT contaning the parameters    *
C*                      for the LBRF approximation.                    *
C*                                                                     *
C*                      And SPEC.DAT for the LBRF approximation due    *
C*                      to a neutron energy spectrum.                  *
C*                                                                     *
C*  Modified from SKYBEAM code by J.K. Shultis and R.E. Faw 7/94       *
C*                        and by Faisal Kahn 6/95, KSU, Manhattan, KS  *
C*                                                                     *
C*                        Output corrected for wall and box 4/96 (JKS) *
C*                        Output table (wall/box) corrected 1/98 (JKS) *
C***********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*32 OUTFIL,INNAM,FD,SPEC
      CHARACTER*23 DUNIT,ETYPE
      COMMON/DOSEU/DUNIT,ETYPE
      COMMON/GAUS01/X(32),W(32)
      COMMON/ECOM/E(30),P(30)
      FD = '(1X,A,$)'
C
C****** PRINT OPENING SCREEN
      WRITE(*,20)
      WRITE(*,21)
C
C****** READ IN COEFFICIENTS FOR APPROXIMATE LBRF
      CALL XBINI
C
C****** ASK USER FOR COMMON PROBLEM DATA
C** Ask if keyboard or file input; Get output file name
      PRINT *
      WRITE(*,FD)'Enter type of data entry (=1 keyboard; =2 file) ==> '
      READ *, INTYPE
      IF(INTYPE.EQ.2) THEN
         PRINT *
         WRITE(*,FD) 'Enter name of input file for skyshine problem data
     1 ===> '
         READ(*,1) INNAM
         OPEN(18,FILE=INNAM)
         READ(18,1,END=99) OUTFIL
      ELSE
         PRINT *
         WRITE(*,FD) 'Enter output filename ==> '
         READ (*,1) OUTFIL
      ENDIF
      ISTRT=0
C

      IF (INTYPE .EQ. 1) THEN
      WRITE(*,15)
      READ*, ISOURCE
        IF (ISOURCE .NE. 1) THEN
      WRITE(*,*)'Enter the energy spectrum filename'
      READ(*,1)SPEC
      OPEN(88,FILE=SPEC)
             READ(88,*)II
             READ(88,*)(E(I),I=1,II)
             READ(88,*)(P(I),I=1,II)
      CLOSE(88)
        ELSE
         II=1
         P(II)=1.0D0
         WRITE(*,17)
         READ*,E(II)
        ENDIF
C
C-- Problem geometry
         WRITE(*,16)
         READ*, IPROB
C-- Air density
         WRITE(*,19)
         READ*,RHO
C-- No. distances
         WRITE(*,22)
         READ(6,*) NPTS
C-- Neutron and Gamma Dose units
         WRITE(*,23)
         READ*,INDX
C-- Order of integration quadrature
         PRINT *
         WRITE(*,FD) 'Enter order for Gauss quadrature (4,8,16,32) ==> '
         READ *, NGAU

C** Read data from input file
      ELSE
            READ(18,*)II
            READ(18,*)(E(I),I=1,II)
            READ(18,*)(P(I),I=1,II)
         READ(18,*,END=99) IPROB,RHO,NPTS,INDX,NGAU
      ENDIF
C
C*** SET CONSTANTS, SET UP QUADRATURE, OPEN OUTPUT FILE
      PI = DACOS(-1.D0)
      CALL READGS(NGAU,X,W)
C
      IF (ISTRT.EQ.0) THEN
         OPEN(12,FILE=OUTFIL,STATUS='UNKNOWN')
         ISTRT=1
      ENDIF
C
      NPTS = NPTS + 1
C
      IF(INDX.EQ.1) THEN
         DUNIT='EDE - ICRP Phantom (AP)'
      ELSEIF(INDX.EQ.2) THEN
         DUNIT='DE - ICRU Sphere (PAR)'
      ELSE
         DUNIT='DE - ICRU Sphere (ISO)'
      END IF

      IF(II .EQ. 1) THEN
         ETYPE='  Monoenergetic'
      ELSE
         ETYPE='       Spectrum'
      ENDIF


C
C********** CALL THE APPROPRIATE GEOMETRY INPUT ROUTINE
      IF(IPROB.EQ.1) THEN
         CALL SILOIO(INDX,RHO,NGAU,NPTS,INTYPE,II)
      ELSEIF (IPROB.EQ.2) THEN
         CALL WALLIO(INDX,RHO,NGAU,NPTS,INTYPE,II)
      ELSE
         CALL BOXIO(INDX,RHO,NGAU,NPTS,INTYPE,II)
      ENDIF
C
C
1     FORMAT(A)
15    format (/' Enter source type:',/
     1         '  (=1 monoenergetic source  )',/
     2         '  (=2 source energy spectrum) ==>',$)
16    FORMAT (/' Enter geometry type for the source containment:',/
     1         '  (=1 source on axis of a circular silo     )',/
     2         '  (=2 source behind an infinitely long wall )',/
     3         '  (=3 source in a rectangular building      )  ==> ',$)
17    FORMAT (/' Enter source energy (.01 < E < 14 MeV)  ==> ',$)
19    FORMAT (/' Enter air density in g/cm^3 (e.g., 0.0012) ==> ',$)
22    FORMAT (/' Enter no. of intermediate distances for calcualtions ==
     1> ',$)
20    FORMAT(22(/),22X,23('*'),/22X,'***** SKYNEUT 1.1 *****'/
     1       22X,23('*')/)
21    FORMAT(' This program evaluates the neutron and secondary-gamma sk
     1yshine doses from'/' produced by a point neutron source, which is
     2either monoenergetic or emits'/' a spectrum of  neutron energies
     3and  which is collimated by one of  three'/' simple geometries:  (
     41) an open silo, (2) a vertical black wall, and (3) a'/' rectangul
     5ar building.   The calculation is based on the integral line-beam'
     6/' skyshine method using a three parameter approximation for the L
     7BRFs.'//)
23    FORMAT (/' Enter the neutron dose unit:',/
     1         '  (=1 AP  Effective dose equivalent)',/
     2         '  (=2 PAR Ambient dose equivalent 10 mm depth)',/
     3         '  (=3 ISO Dose equivalent 10 mm depth)  ==>',$)
C
99    STOP
      END


      SUBROUTINE SILOIO(INDX,RHO,NGAU,N,KEY,ii)
C*****************************************************************
C  Purpose:
C   This subroutine (1) asks for input data for the open silo
C   source, (2) calls for the skyshine dose, and (3) outputs the
C   results.
C
C  Input variables:
C     E   =  source energy
C   RHO   =  air density (g/cm^3)
C  NGAU   =  order of quadrature
C     N   =  no. of intermeduate source-detector distances
C   KEY   =  type of geometry input (=1 keyboard; =2 file)
C
C  Silo geometry variables:
C     R   =  radius of the silo,
C     HS  =  source elevation with respect to top of the silo,
C     HD  =  detector elevation with respect to top of the silo,
C     XD  =  horizontal distance from source to detector Xd
C
C*****************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*23 FD,DUNIT,ETYPE
      COMMON/DOSEU/DUNIT,ETYPE
      COMMON/ECOM/E(30),P(30)
      FD = '(/1X,A,$)'
C
C*** READ GEOMETRY DATA
      IF(KEY.EQ.2) THEN
         READ(18,*,END=41,ERR=41) HS,HD,R,XD
         IF(HS.LE.0.0D0 .OR. R.LE.0.D0 .OR. XD.LE.R) THEN
            WRITE(*,30)
            GOTO 41
         ENDIF
      ELSE
40      WRITE(*,FD)'Enter source elevation below silo top (m) Hs ==> '
        READ*, HS
        WRITE(*,FD)'Enter detector elevation below silo top (m) Hd ==> '
        READ*, HD
        WRITE(*,FD)'Enter silo inner radius (m) R ==> '
        READ*, R
        WRITE(*,FD) 'Enter horizontal source-to-detector distance (m) Xd
     * ==> '
        READ*,XD
        IF(HS.LE.0.0D0 .OR. R.LE.0.D0 .OR. XD.LE.R) THEN
          WRITE(*,30)
          PRINT*,'RE-ENTER GEOMETRY DATA FOR SILO...'
          GOTO 40
        ENDIF
      ENDIF
C
C
C** CALCULATE SOURCE COLLIMATION SOLID ANGLE
42    PI=DACOS(-1.D0)
      W0=HS/DSQRT(R*R + HS*HS)
      OMEGA=2.D0*PI*(1.D0-W0)
C
C*** PRINT GEOMETRY DATA INTO OUTPUT FILE
C
      IF (II .EQ. 1) THEN
      WRITE(12,15)RHO,HS,HD,R,OMEGA,XD,NGAU,ETYPE,E(II)
      WRITE(6,15) RHO,HS,HD,R,OMEGA,XD,NGAU,ETYPE,E(II)
      WRITE(6,*)
      WRITE(12,*)
      WRITE(12,25)DUNIT
      WRITE(6,25)DUNIT
C
      ELSE
      WRITE(12,16)RHO,HS,HD,R,OMEGA,XD,NGAU,ETYPE
      WRITE(6,16) RHO,HS,HD,R,OMEGA,XD,NGAU,ETYPE
      WRITE(12,17)
      WRITE(6,17)
      DO 49 I=1,II
         WRITE(12,7)E(I),P(I)
         WRITE(6,7) E(I),P(I)
49    CONTINUE
7     FORMAT(5X,F10.5,7X,F10.5)
      WRITE(6,*)
      WRITE(6,*)
      WRITE(12,*)
      WRITE(12,*)
      WRITE(12,25)DUNIT
      WRITE(6,25)DUNIT
      ENDIF
C
C
C********* BEGIN CALCULATIONS FOR ALL DISTANCES
      DO 88 I=1,N
         X = (XD/REAL(N))*I
         XI=DATAN((HS-HD)/X)
         DD=DSQRT((HS-HD)**2+X*X)
C        --- Check if detector is inside the silo
         IF (X.LT.R) THEN
            WRITE(12,31)
            WRITE(*,31)
         ENDIF
C        --- Check if source can be seen by detector
           IF ( (HS/R).LE.(HS-HD)/X ) THEN
            WRITE(12,32)
            WRITE(*,32)
         ENDIF
C
         SUMN=0.0
         SUMG=0.0
         DO 10 J=1,II
         ENGY=E(J)
         CALL SILO(INDX,ENGY,RHO,NGAU,W0,DD,XI,DOSEN)
         SUMN=SUMN+DOSEN*P(J)
         CALL SILO(INDX+3,ENGY,RHO,NGAU,W0,DD,XI,DOSEG)
         SUMG=SUMG+DOSEG*P(J)
10       CONTINUE
         DOSEN=SUMN/2.0D0
         DOSEG=SUMG
C
C*** PRINT OUT RESULTS
C        DEN = 100*RHO
         WRITE(12,20)DD,100.D0*DD*RHO,DOSEN,DOSEG
         WRITE(6,20) DD,100.D0*DD*RHO,DOSEN,DOSEG
 88   CONTINUE
      WRITE(12,26)
      WRITE(6,26)
C     WRITE(12,27)
C     WRITE(6,27)
      RETURN
C
C** Error in input data file
41    PRINT*,'**** Error in data file: Program stopped'
      STOP
C
C
15    FORMAT(///'****** SKYNEUT: Silo Geometry '/
     2       '  Air density (g/cm^3)        =',F15.6,/
     3       '  Source elevation (m)        =',F15.2,/
     4       '  Detector elevation (m)      =',F15.2,/
     5       '  Silo radius (m)             =',F15.2,/
     6       '  Source collimation (sr)     =',F15.4,/
     7       '  Source-detector dist. (m)   =',F15.1,/
     8       '  Gauss quadrature order      =',I15,/
     9       '  Source energy type          =',A15,/
     9       '  Source neutron energy (MeV) =',F15.3,/
     9       '  3-parm. approx. LBRF used ',/)
C
C
16    FORMAT(///'****** SKYNEUT: Silo Geometry '/
     2       '  Air density (g/cm^3)        =',F15.6,/
     3       '  Source elevation (m)        =',F15.2,/
     4       '  Detector elevation (m)      =',F15.2,/
     5       '  Silo radius (m)             =',F15.2,/
     6       '  Source collimation (sr)     =',F15.4,/
     7       '  Source-detector dist. (m)   =',F15.1,/
     8       '  Gauss quadrature order      =',I15,/
     9       '  Source energy type          =',A15,/
     9       '  3-parm. approx. LBRF used ',/)
C
17    FORMAT(8X,'ENERGIES',7X,'PROBABILITIES')
20    FORMAT(F12.0,F12.2,2X,2(1PE13.4))
25    FORMAT(10X,'SKYSHINE DOSE: ',A23,/2X,54('-'),/
     1  8X,'S-D',9X,'S-D',7X,'Neutron',5X,'Sec. Gamma',/
     2  8X,'(m)',6X,'(g/cm^2)',4X,'(Sv/neut)',4X,'(Sv/neut)'/
     3  2X,(54('-')))
26    FORMAT(2X,54('-'))
27    FORMAT(2X,'NOTE: ICRP-45 recommends neutron dose be multiplied by
     1',/ 8X,'factor of 2.  Recommendation rejected in the US.')
30    FORMAT(/' **** Error in geometry input data:',/
     1        '      It is required that  (1) Hs > 0',/
     2        '                           (2) R  > 0',/
     3        '                           (3) Xd > R')
31    FORMAT(' **** Warning: Detector inside silo at this distance')
32    FORMAT(' **** Warning: Detector sees source at this distance')
      END


       SUBROUTINE SILO(INDX,E,RHO,NGAU,W0,ZZ,XI,DOSE)
C********************************************************************
C  Purpose:
C   This subroutine calculates the skyshine dose for a source in
C   an open cylindrical silo on the silo's axis.
C
C  Input:
C     E = source energy (MeV)
C   RHO =  air density (g/cm^3)
C  NGAU = Gaussian quadrature order
C    W0 = cosine of the maximun polar angle (silo collimation)
C    ZZ = source-to-detector distance (m)
C    XI = angle between the horizontal axis and source-detector axis
C
C  Output:
C    DOSE = skyshine dose (Sv/neutron)
C
C   NEEDED SUBROUTINES:
C      XBEAM = Approximate LBRF
C     or
C       BEAM = LBRF obtained by point-kernel method,
C********************************************************************
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       COMMON/GAUS01/X(32),W(32)
C
      PI = DACOS(-1.D0)
C*** INTEGRATE LBRF OVER ALL AZIMUTHAL ANGLES 0 TO PI
      TOTSUM = 0.0D0
      DO 100 L = 1,NGAU
         WPSI = 0.5*PI*W(L)
         PSI = 0.5*PI*(X(L) + 1.D0)
C
C*** INTEGRATE OVER POLAR ANGLES
         TOTANS=0.0D0
         DO 99 K = 1,NGAU
           WTHE = 0.5*(1.D0 - W0)*W(K)
           WW = 0.5*(1.D0 - W0)*X(K) + 0.5*(1.D0 + W0)
           IF(WW.GT.1.D0)GOTO 110
           DWW = (1.D0 - WW*WW)**0.5
C*** CALCULATE BEAM EMISSION ANGLE PHI
           DPHI = DCOS(PSI)*DWW*DCOS(XI)+WW*DSIN(XI)
           PHI = DACOS(DPHI)*180./PI
C*** CALCULATE SKYSHINE DOSE FOR GIVEN BEAM
           CALL XBEAM(INDX,E,PHI,ZZ,RHO,RESP)
           TOTANS = TOTANS + WTHE*RESP
99      CONTINUE
C
        TOTSUM = TOTSUM + WPSI*TOTANS
100   CONTINUE
C
      DOSE=0.5*TOTSUM/PI
110   RETURN
      END


      SUBROUTINE WALLIO(INDX,RHO,NGAU,N,KEY,II)
C******************************************************************
C  Purpose:
C   This subroutine (1) asks for input data for a point isotropic
C   monoenergetic source behind an infintely long vertical wall,
C   (2) calls for the skyshine dose, and (3) outputs the results.
C
C  Input variables:
C     E   =  source energy
C   RHO   =  air density (g/cm^3)
C  NGAU   =  order of quadrature
C     N   =  no. of intermeduate source-detector distances
C   KEY   =  type of geometry input (=1 keyboard; =2 file)
C
C  Other subroutine variables:
C     R   =  perped cular source-to-wall distance (m)
C     HS  =  source elevation below top of wall(m),
C     HD  =  detector elevation below top of wall(m),
C     ZD  =  source-detector offset (parallel to top of wall) (m),
C     X   =  horizontal source-to-detector distance (m)
C
c******************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*23 FD,DUNIT,ETYPE
      COMMON/DOSEU/DUNIT,ETYPE
      COMMON/ECOM/E(30),P(30)
      FD = '(/1X,A,$)'
C
C*** READ GEOMETRY DATA
      IF(KEY.EQ.2) THEN
         READ(18,*,END=41,ERR=41) HS,HD,R,ZD,XD
         IF(HS.LE.0.0D0 .OR. R.LE.0.D0 .OR. XD.LE.R) THEN
            WRITE(*,30)
            GOTO 41
         ENDIF
      ELSE
40     WRITE(*,FD)'Enter source elevation below top of wall (m) Hs ==> '
       READ*,HS
       WRITE(*,FD) 'Enter detector elevation below top of wall (m) Hd ==
     1> '
       READ*,HD
       WRITE(*,FD) 'Enter perpendicular distance of source from wall (m)
     1 R ==> '
       READ*,R
       WRITE(*,FD) 'Enter horizontal detector offset from source-wall no
     1rmal (m) Zd ==> '
       READ*,ZD
       WRITE(*,FD)'Enter horiz. source-detector dist. along source-wall
     1normal (m) Xd ==> '
       READ*,XD
       IF(HS.LE.0.0D0 .OR. R.LE.0.D0 .OR. XD.LE.R) THEN
         WRITE(*,30)
         PRINT*,'RE-ENTER GEOMETRY DATA FOR WALL GEOMETRY...'
         GOTO 40
       ENDIF
      ENDIF
      PI=DACOS(-1.D0)
C
C*** PRINT GEOMETRY DATA INTO OUTPUT FILE
C
      IF (II .EQ. 1) THEN
      WRITE(12,15)RHO,HS,HD,R,ZD,XD,NGAU,ETYPE,E(II)
      WRITE(6,15) RHO,HS,HD,R,ZD,XD,NGAU,ETYPE,E(II)
      WRITE(6,*)
      WRITE(12,*)
      WRITE(12,25)DUNIT
      WRITE(6,25)DUNIT
C
      ELSE
      WRITE(12,16)RHO,HS,HD,R,ZD,XD,NGAU,ETYPE
      WRITE(6,16) RHO,HS,HD,R,ZD,XD,NGAU,ETYPE
      WRITE(12,17)
      WRITE(6,17)
      DO 49 I=1,II
         WRITE(12,7)E(I),P(I)
         WRITE(6,7) E(I),P(I)
49    CONTINUE
7     FORMAT(5X,F10.5,7X,F10.5)
      WRITE(6,*)
      WRITE(6,*)
      WRITE(12,*)
      WRITE(12,*)
      WRITE(6,25)DUNIT
      WRITE(12,25)DUNIT
      ENDIF
C
C********* BEGIN CALCULATIONS FOR ALL DISTANCES
       DO 88 I=1,N
         X = (XD / REAL(N))*I
         ZETA=DATAN(ZD/X)
         XI=DATAN((HS-HD)/DSQRT(X*X + ZD*ZD))
         DD=DSQRT((HS-HD)**2 + X*X + ZD*ZD)
C        --- Check if detector is on source side of wall
         IF (X.LT.R) THEN
            WRITE(12,31)
            WRITE(*,31)
         ENDIF
C        --- Check if source can be seen by detector
         IF ( (HS/R).LE.(HS-HD)/X ) THEN
            WRITE(12,32)
            WRITE(*,32)
         ENDIF
C
         SUMN=0.0
         SUMG=0.0
         DO 10 J=1,II
         ENGY=E(J)
         CALL WALL(INDX,ENGY,RHO,NGAU,R,HS,DD,XI,ZETA,DOSEN)
         SUMN=SUMN+DOSEN*P(J)
         CALL WALL(INDX+3,ENGY,RHO,NGAU,R,HS,DD,XI,ZETA,DOSEG)
         SUMG=SUMG+DOSEG*P(J)
10       CONTINUE
         DOSEN=SUMN/2.0D0
         DOSEG=SUMG
C
         WRITE(12,20) X,DD,DOSEN,DOSEG
         WRITE(6,20)  X,DD,DOSEN,DOSEG
88    CONTINUE
      WRITE(12,26)
      WRITE(6,26)
C     WRITE(12,27)
C     WRITE(6,27)
C
      RETURN
C
C** Error in input data file
41    PRINT*,'**** Error in data file: Program stopped'
      STOP
C
C
15    FORMAT(///'****** SKYNEUT: Wall Geometry'/
     2       '  Air density (g/cm^3)         =',F15.6,/
     3       '  Source elevation (m)         =',F15.2,/
     4       '  Detector elevation (m)       =',F15.2,/
     5       '  Source-to-wall distance(m)   =',F15.2,/
     6       '  Detector horiz. offset (m)   =',F15.2,/
     7       '  Source-detector dist. (m)    =',F15.1,/
     8       '  Gauss quadrature order       =',I15,/
     8       '  Source energy type           =',A15,/
     9       '  Source neutron energy        =',F15.3,/
     9       '  3-parm. approx. LBRF used ',/)
C
16    FORMAT(///'****** SKYNEUT: Wall Geometry'/
     2       '  Air density (g/cm^3)         =',F15.6,/
     3       '  Source elevation (m)         =',F15.2,/
     4       '  Detector elevation (m)       =',F15.2,/
     5       '  Source-to-wall distance(m)   =',F15.2,/
     6       '  Detector horiz. offset (m)   =',F15.2,/
     7       '  Source-detector dist. (m)    =',F15.1,/
     8       '  Gauss quadrature order       =',I15,/
     8       '  Source energy type           =',A15,/
     9       '  3-parm. approx. LBRF used ',/)
C
17    FORMAT(8X,'ENERGIES',7X,'PROBABILITIES')
20    FORMAT(F12.0,F12.2,2X,2(1PE13.4))
25    FORMAT(10X,'SKYSHINE DOSE: ',A23,/2X,54('-'),/
     1  8X,' X ',9X,'S-D',7X,'Neutron',5X,'Sec. Gamma',/
     2  8X,'(m)',6X,'   (m)  ',4X,'(Sv/neut)',4X,'(Sv/neut)'/
     3  2X,(54('-')))
26    FORMAT(2X,54('-'))
27    FORMAT(2X,'NOTE: ICRP-45 recommends neutron dose be multiplied by
     1',/ 8X,'factor of 2.  Recommendation rejected in the US.')
30    FORMAT(/' **** Error in geometry input data:',/
     1        '      It is required that  (1) Hs > 0',/
     2        '                           (2) R  > 0',/
     3        '                           (3) Xd > R')
31    FORMAT(' **** Warning: Detector on source side at this distance')
32    FORMAT(' **** Warning: Detector sees source at this distance')
      END


      SUBROUTINE WALL(INDX,E,RHO,NGAU,R,HS,ZZ,XI,ZETA,DOSE)
C*******************************************************************
C Purpose:
C   This subroutine calculates the skyshine dose for the infinite
C   wall problem by integrating the line-beam response function
C   over all directions allowed by the source-wall geometry.
C   All integration are performed by Gaussian quadrature.
C
C  Input:
C     E = source energy (MeV)
C   RHO =  air density (g/cm^3)
C  NGAU = Gaussian quadrature order for the integration
C     R = perpedicular source-to-wall distance (m)
C    HS = source elevation belwow top of wall (m)
C    ZZ = source-to-detector distance along normal from source (m)
C    XI = angle between the horizontal and the source-detector axis
C  ZETA = offset angle (between the normal axis and horizontal axis)
C
C  Output:
C    DOSE = Skyshine Dose (Sv/neutron)
C
C*  NEEDED SUBROUTINES:
C*     XBEAM = Approximate LBRF
C     or:
C*      BEAM = LBRF obtained by point-kernel method,
C********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/GAUS01/X(32),W(32)
C
      FPSI(U,V,Y)=DCOS(U)*DSQRT(1.D0-V*V)*DCOS(Y)+V*DSIN(Y)
      PI=DACOS(-1.D0)
C
C**** For the range 0< PSI < PI/2+ZETA
      SUM12 = 0.0D0
      DO 20 I = 1,NGAU
         B = PI/2.D0 + ZETA
         PSI = 0.5D0*B*(X(I) + 1.D0)
         W1 = 0.5D0*B*W(I)
         SUM11 = 0.0D0
         DO 10 J = 1,NGAU
            A = DATAN( R/(HS*DCOS(PSI - XI)))
            W0 = DCOS(A)
            WW = 0.5D0*(1. - W0)*X(J) + 0.5D0*(1.D0 + W0)
            W2 = 0.5*(1. - W0)*W(J)
            DPHI = FPSI(PSI,WW,XI)
            PHI = DACOS(DPHI)*180.D0/PI
            CALL XBEAM(INDX,E,PHI,ZZ,RHO,RESP)
            SUM11 = SUM11 + W2*RESP
10       CONTINUE
         SUM12 = SUM12 + W1*SUM11
20    CONTINUE
C
C**** For the range PI/2 + ZETA < PSI < 3PI/2 + ZETA
      SUM22 = 0.0D0
      DO 40 I = 1,NGAU
         B = 3*PI/2.D0 + ZETA
         A = PI/2.D0 + ZETA
         PSI = 0.5D0*(B-A)*X(I) + 0.5D0*(B+A)
         W1 = 0.5D0*(B-A)*W(I)
         SUM21 = 0.0D0
         DO 30 J=1,NGAU
            WW = 0.5D0*(X(I)+1.D0)
            DPHI = FPSI(PSI,WW,XI)
            PHI = DACOS(DPHI)*180.D0/PI
            W2 = 0.5D0*W(J)
            CALL XBEAM(INDX,E,PHI,ZZ,RHO,RESP)
            SUM21 = SUM21 + W2*RESP
30       CONTINUE
         SUM22 = SUM22 + W1*SUM21
40    CONTINUE
C
C*** For the range 3 PI/2 + ZETA < PSI < 2 PI
      SUM32 = 0.0D0
      DO 60 I = 1,NGAU
         B = 2.D0*PI
         A = 1.5D0*PI + ZETA
         PSI = 0.5D0*(B-A)*X(I)+0.5D0*(B+A)
         W1 = 0.5D0*(B-A)*W(I)
         SUM31 = 0.0D0
         DO 50 J = 1,NGAU
            A = DATAN( R/(HS*DCOS(PSI-XI)))
            W0 = DCOS(A)
            WW = 0.5*(1.D0-W0)*X(J) + 0.5*(1.+W0)
            W2 = 0.5*(1.D0-W0)*W(J)
            DPHI = FPSI(PSI,WW,XI)
            PHI = DACOS(DPHI)*180./PI
            CALL XBEAM(INDX,E,PHI,ZZ,RHO,RESP)
            SUM31 = SUM31 + W2*RESP
50       CONTINUE
         SUM32 = SUM32 + W1*SUM31
60    CONTINUE
C
C**** ADD THREE COMPONENTS TO GET TOTAL SKYSHINE DOSE
      DOSE = SUM12 + SUM22 + SUM32
      DOSE = DOSE/4.D0/PI
      RETURN
      END
C
C
      SUBROUTINE BOXIO(INDX,RHO,NGAU,N,KEY,II)
C******************************************************************
C
C  Purpose:
C   This subroutine (1) asks for input data for the source in a
C   rectangular building problem, (2) calls for the skyshine dose,
C   and (3) outputs the results.
C
C  Input variables:
C     E   =  source energy
C   RHO   =  air density (g/cm^3)
C  NGAU   =  order of quadrature
C     N   =  no. of intermeduate source-detector distances
C   KEY   =  type of geometry input (=1 keyboard; =2 file)
C
C  Box geometry variables:
C     HS  =  source elevation below top of building (m)
C     HD  =  detector elevation below top of building (m)
C     X1  =  perpendicular distance from source to rear wall (m)
C     X2  =  perpendicular distance from source to front wall (m)
C     Y1  =  perpendicular distance from source to left wall (m)
C     Y2  =  perpendicular distance from source to right wall (m)
C     Z   =  source-detector offset (parallel to top of wall)(m)
C     X   =  horizontal source-to-detector distance (m)
C
c******************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION PSI(5)
      CHARACTER*23 FD,DUNIT,ETYPE
      COMMON/DOSEU/DUNIT,ETYPE
      COMMON/ECOM/E(30),P(30)
      FD = '(/1X,A,$)'
C
C*** READ GEOMETRY DATA
      IF(KEY.EQ.2) THEN
         READ(18,*,END=41,ERR=41) HS,HD,X1,X2,Y1,Y2,YD,XD
         IF(HS.LE.0.0D0.OR.XD.LE.0.D0.OR.X1.LE.0.D0.OR.X2.LE.0.D0.OR.
     1     Y1.LE.0.D0 .OR. Y2.LE.0.D0 .OR. YD.LT.-Y2 .OR. YD.GT.Y1)THEN
            WRITE(*,30)
            GOTO 41
         ENDIF
      ELSE
40      WRITE(*,FD)'Enter source elevation below top of building (m) Hs
     *==> '
        READ*,HS
        WRITE(*,FD)'Enter detector elevation below building top (m) Hd =
     *=> '
        READ*,HD
        WRITE(*,FD)'Enter perp. distance from source to rear wall (m) X1
     * ==> '
        READ*,X1
        WRITE(*,FD)'Enter perp. distance from source to front wall (m) X
     *2 ==> '
        READ*,X2
        WRITE(*,FD)'Enter perp. distance from source to left wall (m) Y1
     * ==> '
        READ*,Y1
        WRITE(*,FD)'Enter perp. distance from source to right wall (m) Y
     *2 ==> '
        READ*,Y2
        WRITE(*,FD)'Enter detector offset from normal to front wall (m)
     *Yd ==> '
        READ*,YD
        WRITE(*,FD)'Enter source-detector dist. along normal to front wa
     *ll (m) Xd ==> '
        READ*,XD

        IF(HS.LE.0.0D0.OR.XD.LE.0.D0.OR.X1.LE.0.D0.OR.X2.LE.0.D0.OR.
     1    Y1.LE.0.D0 .OR. Y2.LE.0.D0 .OR. YD.LT.-Y2 .OR. YD.GT.Y1)THEN
          WRITE(*,30)
          PRINT*,'RE-ENTER GEOMETRY DATA FOR RECTANGULAR BUILDING...'
          GOTO 40
        ENDIF
      ENDIF
      PI = DACOS(-1.D0)
C
C*** PRINT GEOMETRY DATA INTO OUTPUT FILE
C
      IF (II .EQ. 1) THEN
        WRITE(12,15)RHO,HS,HD,X1,X2,Y1,Y2,YD,XD,NGAU,ETYPE,E(II)
        WRITE(6,15) RHO,HS,HD,X1,X2,Y1,Y2,YD,XD,NGAU,ETYPE,E(II)
        WRITE(6,*)
        WRITE(12,*)
        WRITE(12,25)DUNIT
        WRITE(6,25)DUNIT
C
      ELSE
        WRITE(12,16)RHO,HS,HD,X1,X2,Y1,Y2,YD,XD,NGAU,ETYPE
        WRITE(6,16) RHO,HS,HD,X1,X2,Y1,Y2,YD,XD,NGAU,ETYPE
        WRITE(12,17)
        WRITE(6,17)
        DO 49 I=1,II
          WRITE(12,7)E(I),P(I)
          WRITE(6,7) E(I),P(I)
49      CONTINUE
7       FORMAT(5X,F10.5,7X,F10.5)
        WRITE(6,*)
        WRITE(6,*)
        WRITE(12,*)
        WRITE(12,*)
        WRITE(12,25)DUNIT
        WRITE(6,25)DUNIT
      ENDIF
C
C********* BEGIN CALCULATIONS FOR ALL DISTANCES
      DO 88 I=1,N
         X = (XD / REAL(N))*I
         ZETA=DATAN(YD/(X1 + X ))
         PSI(1) = DATAN(Y1/X2)-ZETA
         PSI(2) = 0.5*PI+DATAN(X1/Y1)-ZETA
         PSI(3) = PI+DATAN(Y2/X1)-ZETA
         PSI(4) = 2.D0*PI-DATAN(Y2/X2)-ZETA
         PSI(5) = -DATAN(Y2/X2)-ZETA
         XI=DATAN((HS-HD)/DSQRT(X*X + YD*YD))
         DD=DSQRT((HS-HD)**2 + X*X + YD*YD)
C        --- Check if detector is inside building
         IF (X.LT.X2) THEN
            WRITE(12,31)
            WRITE(*,31)
         ENDIF
C        --- Check if source can be seen
         IF ( (HS/X2).LE.(HS-HD)/(X2+XD) ) THEN
            WRITE(12,32)
            WRITE(*,32)
         ENDIF
C
         SUMN=0.0
         SUMG=0.0
         DO 10 J=1,II
         ENGY=E(J)
      CALL BOX(INDX,ENGY,RHO,NGAU,X1,X2,Y1,Y2,HS,DD,XI,PSI,ZETA,DOSEN)
         SUMN=SUMN+DOSEN*P(J)
      CALL BOX(INDX+3,ENGY,RHO,NGAU,X1,X2,Y1,Y2,HS,DD,XI,PSI,ZETA,DOSEG)
         SUMG=SUMG+DOSEG*P(J)
10       CONTINUE
         DOSEN=SUMN/2.0D0
         DOSEG=SUMG
C
        WRITE(12,20) X,DD,DOSEN,DOSEG
        WRITE(6,20)  X,DD,DOSEN,DOSEG
88    CONTINUE
      WRITE(12,26)
      WRITE(6,26)
C     WRITE(12,27)
C     WRITE(6,27)
C
C
      RETURN
C
C** Error in input data file
41    PRINT*,'**** Error in data file: Program stopped'
      STOP
C
C
15    FORMAT(///'****** SKYNEUT: Rectangular Building Geometry'/
     2       '  Air density (g/cm^3)             =',F15.6,/
     3       '  Source elevation (m)             =',F15.2,/
     3       '  Detector elevation (m)           =',F15.2,/
     4       '  Source-rear wall distance (m)    =',F15.2,/
     4       '  Source-front wall distance (m)   =',F15.2,/
     4       '  Source-left wall distance (m)    =',F15.2,/
     4       '  Source-right wall distance (m)   =',F15.2,/
     5       '  Detector offset from normal (m)  =',F15.2,/
     6       '  Detector-front wall distance(m)  =',F15.2,/
     8       '  Gauss quadrature order           =',I15,/
     8       '  Source energy type               =',A15,/
     9       '  Source neutron energy            =',F15.3,/
     9       '  3-parm. approx. LBRF used ',/)
C
16    FORMAT(///'****** SKYNEUT: Rectangular Building Geometry'/
     2       '  Air density (g/cm^3)             =',F15.6,/
     3       '  Source elevation (m)             =',F15.2,/
     3       '  Detector elevation (m)           =',F15.2,/
     4       '  Source-rear wall distance (m)    =',F15.2,/
     4       '  Source-front wall distance (m)   =',F15.2,/
     4       '  Source-left wall distance (m)    =',F15.2,/
     4       '  Source-right wall distance (m)   =',F15.2,/
     5       '  Detector offset from normal (m)  =',F15.2,/
     6       '  Detector-front wall distance(m)  =',F15.2,/
     8       '  Gauss quadrature order           =',I15,/
     8       '  Source energy type               =',A15,/
     9       '  3-parm. approx. LBRF used ',/)
C
17    FORMAT(8X,'ENERGIES',7X,'PRPBABILITIES')
20    FORMAT(F12.0,F12.2,2X,2(1PE13.4))
25    FORMAT(10X,'SKYSHINE DOSE: ',A23,/2X,54('-'),/
     1  8X,' X ',9X,'S-D',7X,'Neutron',5X,'Sec. Gamma',/
     2  8X,'(m)',6X,'   (m)  ',4X,'(Sv/neut)',4X,'(Sv/neut)'/
     3  2X,(54('-')))
26    FORMAT(2X,54('-'))
27    FORMAT(2X,'NOTE: ICRP-45 recommends neutron dose be multiplied by
     1',/ 8X,'factor of 2.  Recommendation rejected in the US.')
30    FORMAT(/' **** Error in geometry input data:',/
     1        '      It is required that  (1) Hs > 0',/
     2        '                           (2) Xd > 0',/
     3        '                           (3) X1,X2 > 0' ,/
     4        '                           (4) Y1,Y2 > 0' ,/
     5        '                           (5) -Y2 < Yd < Y1')
31    FORMAT(' **** Warning: Detector inside building at this distance')
32    FORMAT(' **** Warning: Detector sees source at this distance')
       END


      SUBROUTINE BOX(INDX,E,RHO,NGAU,X1,X2,Y1,Y2,HS,ZZ,XI,PSI,ZETA,DOSE)
C****************************************************************
C Purpose:
C   This subroutine calculates the skyshine dose from a point
C   isotropic monoenergetic source located in a rectangular
C   building.  The dose is calculated by numerically integrating
C   the line beam response function over all allowed emission
C   directions.
C
C  Input:
C      E  =  source energy (MeV)
C   NGAU  =  Gaussian quadrature order
C     X1  =  perpendicular distance from source to rear wall (m)
C     X2  =  perpendicular distance from source to front wall (m)
C     Y1  =  perpendicular distance from source to left wall (m)
C     Y2  =  perpendicular distance from source to right wall (m)
C     HS  =  source elevation with respect to top of wall (m)
C     ZZ  =  Source-to-detector distance (m)
C     XI  =  Angle between horiz. and source-detector axis (deg.)
C    PSI  =  Azimuthal angles defined in source plane (deg.)
C    ZETA =  Offset angle (between normal axis and horiz. axis)(deg)
C
C  Output:
C    DOSE = Skyshine Dose
C
C   NEEDED SUBROUTINES:
C      XBEAM = Approximate LBRF
C     or:
C       BEAM = LBRF obtained by point-kernel method
C****************************************************************
C
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       COMMON/GAUS01/X(32),W(32)
       DIMENSION PSI(5)

      FPSI(U,V,Y)=DCOS(U)*DSQRT(1.D0-V*V)*DCOS(Y)+V*DSIN(Y)
      PI = DACOS(-1.D0)
C
C**** for the range PSI(5) < PSI < PSI(1)
      SUM12 = 0.0D0
      DO 20 I = 1,NGAU
        A = PSI(5)
        B = PSI(1)
        PPSI = 0.5D0*((B-A)*X(I) + (B+A))
        W1 = 0.5D0*(B-A)*W(I)
        SUM11 = 0.D0
        DO 10 J = 1,NGAU
           THETA1 = DATAN(X2/(HS*DCOS(PPSI + ZETA)))
           W0 = DCOS(THETA1)
           WW = 0.5D0*((1.D0-W0)*X(J) + (1.D0+W0))
           W2 = 0.5D0*(1.D0-W0)*W(J)
           DPHI = FPSI(PPSI,WW,XI )
           PPHI = DACOS(DPHI)*180.D0/PI
           CALL XBEAM(INDX,E,PPHI,ZZ,RHO,RESP)
           SUM11 = SUM11 + W2*RESP
 10     CONTINUE
        SUM12 = SUM12 + W1*SUM11
20    CONTINUE
C
C**** For the range PSI(2) < PSI < PSI(1)
      SUM22 = 0.D0
      DO 40 I = 1,NGAU
        A = PSI(1)
        B = PSI(2)
        PPSI = 0.5D0*((B-A)*X(I) + (B+A))
        W1 = 0.5D0*(B-A)*W(I)
        SUM21 = 0.D0
        DO 30 J = 1,NGAU
           THETA2 = DATAN(Y1/(HS*DSIN(PPSI+ZETA)))
           W0 = DCOS(THETA2)
           WW = 0.5D0*((1.D0-W0)*X(J) + (1.D0+W0))
           W2 = 0.5*(1.D0 - W0)*W(J)
           DPHI = FPSI(PPSI,WW,XI)
           PPHI = DACOS(DPHI)*180.D0/PI
           CALL XBEAM(INDX,E,PPHI,ZZ,RHO,RESP)
           SUM21 = SUM21 + W2*RESP
 30     CONTINUE
        SUM22 = SUM22 + W1*SUM21
 40   CONTINUE
C
C**** For the range PSI(3) < PSI < PSI(2)
      SUM32 = 0.D0
      DO 60 I = 1,NGAU
        A = PSI(2)
        B = PSI(3)
        PPSI = 0.5D0*((B-A)*X(I) + (B+A))
        W1 = 0.5D0*(B-A)*W(I)
        SUM31 = 0.D0
        DO 50 J = 1,NGAU
           THETA3 = DATAN(X1/(-HS*DCOS(PPSI+ZETA)))
           W0 = DCOS(THETA3)
           WW = 0.5D0*((1.D0-W0)*X(J) + (1.D0+W0))
           W2 = 0.5D0*(1.D0-W0)*W(J)
           DPHI = FPSI(PPSI,WW,XI)
           PPHI = DACOS(DPHI)*180.D0/PI
           CALL XBEAM(INDX,E,PPHI,ZZ,RHO,RESP)
           SUM31 = SUM31 + W2*RESP
 50     CONTINUE
        SUM32 = SUM32 + W1*SUM31
 60   CONTINUE
C
C**** For the range PSI(4) < PSI < PSI(3)
      SUM42 = 0.D0
      DO 80 I = 1,NGAU
        A = PSI(3)
        B = PSI(4)
        PPSI = 0.5D0*((B-A)*X(I) + (B+A))
        W1 = 0.5D0*(B-A)*W(I)
        SUM41 = 0.D0
        DO 70 J = 1,NGAU
           THETA4 = DATAN(Y2/(-HS*DSIN(PPSI+ZETA)))
           W0 = DCOS(THETA4)
           WW = 0.5D0*((1.D0-W0)*X(J) + (1.D0+W0))
           W2 = 0.5D0*(1.D0-W0)*W(J)
           DPHI = FPSI(PPSI,WW, XI)
           PPHI = DACOS(DPHI)*180.D0/PI
           CALL XBEAM(INDX,E,PPHI,ZZ,RHO,RESP)
           SUM41 = SUM41 + W2*RESP
 70     CONTINUE
        SUM42 = SUM42 + W1*SUM41
 80   CONTINUE
C
C**** Sum four components to get total skyshine dose
      DOSE = SUM12 + SUM22 + SUM32 + SUM42
      DOSE = 0.25D0*DOSE/PI
      RETURN
      END



      BLOCK DATA EAGRID
C*************************************************************************
C  Initialize data for XBEAM
C*************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /VECC/ EEX(14),PPSIX(18)
C***** Discrete angles for calculation of approximate Neut. and Gamma lbrf
        DATA (PPSIX(I),I=1,18)
     *       /1.0,2.0,4.0,6.0,8.0,12.0,18.0,25.0,35.0,45.0,
     *        55.0,65.0,75.0,90.0,110.0,130.0,150.0,170.0/
C***** Discrete energies for calculation of approximate Neut. and Gamma lbrf
        DATA (EEX(I),I=1,14)
     *       /0.01,0.05,0.1,0.5,1.0,1.5,2.0,2.4,3.0,5.0,
     *        7.0,9.0,11.0,14.0/
       END


      SUBROUTINE XBINI
C**********************************************************************
C Reads data files containing coefficients for the three-paramter
C approximation to the LBRF.  Call once before using subprogram XBEAM.
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*15 EHDR
      COMMON /DATT/ G(6,14,18,3),ANGL(18)
C
C ****** Read in the fit parameters for the approximate LBRF
      OPEN(21,FILE='nede.dat')
      OPEN(23,FILE='gede.dat')

      OPEN(25,FILE='ndep.dat')
      OPEN(27,FILE='gdep.dat')

      OPEN(29,FILE='ndei.dat')
      OPEN(31,FILE='gdei.dat')

C** Read Neutron and Gamma Parameters (AP)
      DO 20 I=1,14
        READ(21,10) EHDR
        READ(21,*)(ANGL(J),G(1,I,J,1),G(1,I,J,2),G(1,I,J,3),J=1,18)
20    CONTINUE

      DO 30 I=1,14
        READ(23,10) EHDR
        READ(23,*)(ANGL(J),G(4,I,J,1),G(4,I,J,2),G(4,I,J,3),J=1,18)
30    CONTINUE

C** Read Neutron and Gamma Parameters (PAR)
      DO 40 I=1,14
        READ(25,10) EHDR
        READ(25,*)(ANGL(J),G(2,I,J,1),G(2,I,J,2),G(2,I,J,3),J=1,18)
40    CONTINUE

      DO 50 I=1,14
        READ(27,10) EHDR
        READ(27,*)(ANGL(J),G(5,I,J,1),G(5,I,J,2),G(5,I,J,3),J=1,18)
50    CONTINUE

C** Read Neutron and Gamma Parameters (ISO)
      DO 60 I=1,14
        READ(29,10) EHDR
        READ(29,*)(ANGL(J),G(3,I,J,1),G(3,I,J,2),G(3,I,J,3),J=1,18)
60    CONTINUE

      DO 70 I=1,14
        READ(31,10) EHDR
        READ(31,*)(ANGL(J),G(6,I,J,1),G(6,I,J,2),G(6,I,J,3),J=1,18)
70    CONTINUE

10    FORMAT(A)
      CLOSE(21)
      CLOSE(23)
      CLOSE(25)
      CLOSE(27)
      CLOSE(29)
      CLOSE(31)
      RETURN
      END



      SUBROUTINE XBEAM(INDX,E,PSI,ZZ,RHOACT,RESP)
C***************************************************************************
C  Purpose:
C    This subroutine calculates the skyshine dose at a detector
C    arising from a monoenergetic and monodirectional neutron beam
c    using a 3-parameter approximation for the LBRF.
C
C  Input:
C      E = Source energy (MeV),
C    PSI = neutron emission angle (deg.)
C     ZZ = source-to-detector distance (m)
C RHOACT = air density (g cm^-3)
C
C  Output:
C     RESP = Line-beam response function (Sv/neutron)
C
C  Variables:
C        FK     = flux-to-dose conversion factor
C        IE     = energy group index in which energy E resides
C        IEADJ  = adjacent enrgy index
C        JP     = angular group index in which a beam angle reside
C        JPADJ  = adjacent angle index
C
C  Needed Routines:  XBINI to read LBRF approximation coefficients
C                    EAGRID Block Data to set up E and ANG grid
C
C**************************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        DIMENSION  FF(20),FIE(20),FIEADJ(20)
        COMMON /DATT/ G(6,14,18,3),ANGL(18)
        COMMON /VECC/ EE(14),PPSI(18)

          NE = 14
          NANG = 18

        RHOAIR = 0.0012D0
        RFACT = RHOACT/RHOAIR

C** Find energy indices IE and IEADJ bracketing energy E

        IF (E.LE.EE(1)) THEN
            IE = 1
        ELSEIF (E.GE.EE(NE)) THEN
            IE = NE
        ELSE
            DO 10 I=2,NE
              IF(E.GE.EE(I-1).AND.E.LT.EE(I)) THEN
                 IE = I-1
                 GOTO 11
              ENDIF
  10        CONTINUE
        ENDIF
  11    EIE = EE(IE)
        IEADJ = IE+1
        IF(IE.EQ.NE) IEADJ = NE-1

C*** Find angular indices JP and JPADJ bracketing angle PSI

        IF (PSI.LE.PPSI(1)) THEN
            JP = 1
        ELSEIF (PSI.GE.PPSI(NANG)) THEN
            JP = NANG
        ELSE
            DO 20 I=2,NANG
              IF(PSI.GE.PPSI(I-1).AND.PSI.LT.PPSI(I)) THEN
                 JP = I-1
                 GOTO 21
              ENDIF
  20        CONTINUE
        ENDIF
  21    JPADJ = JP + 1
        IF (JP.EQ.NANG) JPADJ = NANG-1

C** Evaluate LBRF at four surrounding tabulated grid points
        X=ZZ*RFACT
        FIE(JP)=DEXP(G(INDX,IE,JP,1) + G(INDX,IE,JP,2)*X)
     *          *X**G(INDX,IE,JP,3)
        FIEADJ(JP)=DEXP(G(INDX,IEADJ,JP,1)+G(INDX,IEADJ,JP,2)*X)
     *             *X**G(INDX,IEADJ,JP,3)
        FIE(JPADJ)=DEXP(G(INDX,IE,JPADJ,1)+G(INDX,IE,JPADJ,2)*X)
     *             *X**G(INDX,IE,JPADJ,3)
        FIEADJ(JPADJ)=DEXP(G(INDX,IEADJ,JPADJ,1)
     *                                + G(INDX,IEADJ,JPADJ,2)*X)
     *             *X**G(INDX,IEADJ,JPADJ,3)
C
C** Interpolate between bracketing energies
        EFACT=(E-EE(IE))/(EE(IEADJ)-EE(IE))
        FF(JP)=FIE(JP)+(FIEADJ(JP)-FIE(JP))*EFACT
        FF(JPADJ)=FIE(JPADJ)+(FIEADJ(JPADJ)-FIE(JPADJ))*EFACT

C** Log-interpolate between the bracketing angles
        FF(JP)=DLOG(FF(JP))
        FF(JPADJ)=DLOG(FF(JPADJ))
        RESP=FF(JP)+(FF(JPADJ)-FF(JP))*(PSI-PPSI(JP))/
     *              (PPSI(JPADJ)-PPSI(JP))

        RESP=E*(RFACT*RFACT)*DEXP(RESP)
        RETURN
      END



      SUBROUTINE READGS(NGAU,X,W)
C*************************************************************
C  Routine returns data needed for Gaussian quadrature
C
C  Input:
C    NGAU = Order of quadrature (4, 8, 16, or 32)
C  Return:
C      X(NGAU), W(NGAU) : Quadrature ordinates and weights
C*************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(32),W(32),GAUS(30,2)

      DATA (GAUS(I,1),I=1,2)/ .861136311594053, .339981043584856 /
      DATA (GAUS(I,2),I=1,2)/ .347854845137454, .652145154862546 /
      DATA (GAUS(I,1),I=3,6)/ .960289856497536, .796666477413627,
     1                        .525532409916329, .183434642495650/
      DATA (GAUS(I,2),I=3,6)/ .101228536290376, .222381034453374,
     1                        .313706645877887, .362683783378362 /
      DATA(GAUS(I,1),I=7,14)/ .095012509837637, .281603550779259,
     1      .458016777657227, .617876244402644, .755404408355003,
     2      .865631202387832, .944575023073233, .989400934991649/
      DATA(GAUS(I,2),I=7,14)/ .189450610455068, .182603415044924,
     1      .169156519395003, .149595988816577, .124628971255534,
     2      .095158511682493, .062253523938648, .027152459411754/
      DATA(GAUS(I,1),I=15,30)/.997263861849482, .985611511545268,
     1                        .964762255587506, .934906075937740,
     2                        .896321155766052, .849367613732570,
     3                        .794483795967942, .732182118740290,
     4                        .663044266930215, .587715757240762,
     5                        .506899908932229, .421351276130635,
     6                        .331868602282128, .239287362252137,
     7                        .144471961584796, .048307665687738/
      DATA(GAUS(I,2),I=15,30)/.007018610009470, .016274394730906,
     1                        .025392065309262, .034273862913021,
     2                        .042835898022227, .050998059262376,
     3                        .058684093478536, .065822222776362,
     4                        .072345794108849, .078193895787070,
     5                        .083311924226947, .087652093004404,
     6                        .091173878695763, .093844399080805,
     7                        .095638720079275, .096540088514728 /
C
C*** Read in quadrature ordinates and weights and use symmetry
      IF (NGAU.EQ.4) THEN
         DO 5 I = 1,2
           X(5-I) = GAUS(I,1)
5        W(5-I) = GAUS(I,2)
         DO 10 I = 1, 2
           X(I) = -X(5-I)
10       W(I) = W(5-I)
      ELSEIF (NGAU.EQ.8) THEN
         DO 20 I = 1,4
           X(9-I) = GAUS(I+2,1)
20       W(9-I) = GAUS(I+2,2)
         DO 30 I = 1 , 4
           X(I) = -X(9-I)
30         W(I) = W(9-I)
      ELSEIF(NGAU.EQ.16)THEN
         DO 40 I = 1,8
           X(8+I) = GAUS(I+6,1)
40       W(8+I) = GAUS(I+6,2)
         DO 50 I = 9, 16
           X(17-I) = -X(I)
50       W(17-I) = W(I)
      ELSE
         DO 60 I = 1,16
           X(33-I) = GAUS(I+14,1)
 60      W(33-I) = GAUS(I+14,2)
         DO 70 I = 1,16
           X(I) = -X(33-I)
70       W(I) = W(33-I)
       ENDIF
       RETURN
       END
